function Z = arytmetyka_obrazow(varargin)

args = parse_inputs(varargin{:});

klasaX = class(args.X);
klasaY = class(args.Y);

if isempty(args.klasa)
    if isinteger(args.X) && isinteger(args.Y)
        if intmax(klasaX)<=intmax(klasaY)
            args.klasa = klasaX;
        else
            args.klasa = klasaY;
        end;
    elseif isfloat(args.Y)
        args.klasa = klasaX;
    else
        args.klasa = klasaY;
    end;   
elseif ~strcmp(args.klasa, class(args.X))
    s = ['args.X = ' args.klasa '(args.X);'];
    eval(s);    
elseif ~strcmp(args.klasa, class(args.Y))
    s = ['args.Y = ' args.klasa '(args.Y);'];
    eval(s);    
end;
% if strcmp(args.przepelnienie, 'przewiniecie')
%        if ~isfloat(args.X)
%            args.X = double(args.X) + 1;
%        end;
%        if ~isfloat(args.Y)
%            args.Y = double(args.Y) + 1;
%        end;
% end;

if strcmp(args.przepelnienie, 'przewiniecie')
       if ~isfloat(args.X)
           args.X = double(args.X);
       end;
       if ~isfloat(args.Y)
           args.Y = double(args.Y);
       end;
end;

args.Z = zeros(size(args.X));

switch args.operator
    case '+'
        args.Z = suma(args.X, args.Y);
    case '-'
        args.Z = roznica(args.X, args.Y);
    case '|-|'
        args.Z = abs_roznica(args.X, args.Y); 
    case '.*'
        args.Z = iloczyn(args.X, args.Y);
    case './'
        args.Z = iloraz(args.X, args.Y);
end;

if strcmp(args.przepelnienie, 'przewiniecie')
    args.Z = przewin(args.Z, args.klasa);
end;

Z = args.Z;

if ~strcmp(args.klasa, class(args.Z))
    s = ['Z = ' args.klasa '(Z);'];
    eval(s);
end;

%--------------------------------------------------
function Z = suma(X, Y)

Z = X + Y;
    
%--------------------------------------------------
function Z = roznica(X, Y)

Z = X - Y;


%--------------------------------------------------
function Z = abs_roznica(X, Y)

Z = abs(X - Y);

%--------------------------------------------------
function Z = iloczyn(X, Y)

Z = X .* Y;

%--------------------------------------------------
function Z = iloraz(X, Y)

Z = X ./ Y;

%--------------------------------------------------
function Z = przewin(Z, klasa)

if ~isempty(strfind(klasa,'int'))
    if isinteger(Z)
        Z = mod(Z, intmax(klasa));
    else
        Z = mod(Z, double(intmax(klasa))+1);%Z = mod(Z, double(intmax(klasa))+1) - 1;
    end;
else
    Z = mod(Z, realmax(klasa));
end;


%--------------------------------------------------
function args = parse_inputs(varargin)

iptchecknargin(1,Inf,nargin,mfilename);

args.oprator = '';
args.przepelnienie = 'saturacja';
args.klasa='';

[args.X args.Y]= check_XY(varargin{1},varargin{2});

if rem(nargin,2) == 0
    first_prop_arg = 3;
    for k = first_prop_arg:2:nargin
        prop_string = check_property_string(varargin{k});
        switch prop_string
        case 'operator'
            args.operator = check_operator(varargin{k+1});  
        case 'przepelnienie'
            args.przepelnienie = check_przepelnienie(varargin{k+1});
        case 'klasa'
            args.klasa = check_klasa(varargin{k+1});
        otherwise
            eid = sprintf('Images:%s:internalError',mfilename);
            msg = sprintf('Unrecognized property string: %s', ...
                           prop_string);
            error(eid,'%s',msg);
        end;
    end;
end; 

%--------------------------------------------------
function prop_string = check_property_string(prop_string)

if ~ischar(prop_string)
    eid = sprintf('Images:%s:invalidPropertyName',mfilename);
    msg = 'Property names must be strings.';
    error(eid,'%s',msg);
end
valid_strings = {'operator'
                 '+'
                 '-'
                 '|-|'
                 '.*'
                 './'
                 'przepelnienie'
                 'saturacja'
                 'przewiniecie'
                 'klasa'
                 'uint8'
                 'uint16'
                 'uint32'
                 'uint64'
                 'int8'
                 'int16'
                 'int32'
                 'int64'
                 'single'
                 'double'};
idx = strmatch(lower(prop_string), valid_strings);
switch length(idx)
 case 0
    eid = sprintf('Images:%s:unknownPropertyName',mfilename);
    msg = sprintf('Unrecognized property name: ''%s''', prop_string);
    error(eid,'%s',msg);
  case 1
    prop_string = valid_strings{idx};
  otherwise
    eid = sprintf('Images:%s:ambiguousPropertyName',mfilename);
    msg = sprintf('Ambiguous property name: ''%s''', prop_string);
    error(eid,'%s',msg);
end

%--------------------------------------------------
function [X Y] = check_XY(X, Y)

if (~isnumeric(X) && ~islogical(X)) || issparse(X)
    eid = sprintf('Images:%s:invalidImage',mfilename);
    msg = 'X and Y must be a nonsparse numeric array.';
    error(eid,'%s',msg);
end

sizeX = size(X);
sizeY = size(Y);

if ~isequal(sizeX, sizeY) && (numel(X) ~= 1 && numel(Y) ~= 1)
    eid = sprintf('Images:%s:invalidImage',mfilename);
    msg = 'X and Y dimensions must agree or X or Y must be a scalar.';
    error(eid,'%s',msg);
end

%--------------------------------------------------
function operator = check_operator(operator)

if ~ischar(operator)
    eid = sprintf('Images:%s:invalidImage',mfilename);
    msg = 'Operator must be a string.';
    error(eid,'%s',msg);
end

operator = check_property_string(operator);


%--------------------------------------------------
function przepelnienie = check_przepelnienie(przepelnienie)

if ~ischar(przepelnienie)
    eid = sprintf('Images:%s:invalidImage',mfilename);
    msg = 'Przepenienie must be a string.';
    error(eid,'%s',msg);
end

przepelnienie = check_property_string(przepelnienie);

%--------------------------------------------------
function klasa = check_klasa(klasa)

if ~ischar(klasa)
    eid = sprintf('Images:%s:invalidImage',mfilename);
    msg = 'Klasa must be a string.';
    error(eid,'%s',msg);
end

klasa = check_property_string(klasa);