function Y = przeksztalc_obraz_new(varargin)

args = parse_inputs(varargin{:});

[w z] = wspolrzedne(args.X);

switch args.typ
    case 'Tform'
        xys=[1 1];
        tform = maketform('affine', args.wymiar);
        Y = imtransform(args.X,tform,...
                        'XData',[1 size(args.X,2) + args.wymiar(3,1)],...
                        'YData',[1 size(args.X,1) + args.wymiar(3,2)],...
                        'size', size(args.X),'XYScale', xys);
    otherwise
        switch args.typ
            case 'translacja'
                [x y] = translacja(w, z, args.wymiar(1), args.wymiar(2),...
                                   args.kierunek);
            case 'skalowanie'
                [x y] = skalowanie(w, z, args.wymiar(1), args.wymiar(2),...
                                   args.kierunek);
            case 'obrot'
                [x y] = obrot(w, z, args.wymiar(3), args.wymiar(1),...
                              args.wymiar(2), args.kierunek);
        end;

        [x y] = dopasuj_wspolrzedne(x, y);

        Y = tworz_obraz(args.X, x, y, args.kierunek, args.wypelnienie);
end;

%--------------------------------------------------
function [x y] = dopasuj_wspolrzedne(w, z)

x = round(w);
y = round(z);
x(x<0) = 0;
y(y<0) = 0;
x(x>size(z,2)) = 0;
y(y>size(w,1)) = 0;

%--------------------------------------------------
function Y = tworz_obraz(X, x, y, kierunek, wypelnienie)

Y = ones(size(X))*wypelnienie;

if kierunek == 1
    for i=1:size(x,1)
        for j=1:size(y,2)
            if x(i,j)*y(i,j)~=0
                Y(y(i,j),x(i,j))=X(i,j);
            end;
        end;
    end;
else
    for i=1:size(x,1)
        for j=1:size(y,2)
            if x(i,j)*y(i,j)~=0
                Y(i,j)=X(y(i,j),x(i,j));
            end;
        end;
    end;
end;

%--------------------------------------------------
function [x y] = obrot(w, z, alpha, x0, y0, kierunek)

x = zeros(size(z,1),size(w,2));
y = zeros(size(z,1),size(w,2));

x = round(((w-1-x0)*cos(alpha) - (z-1-y0)*kierunek*sin(alpha)) + 1 + x0);
y = ((w-1-x0)*kierunek*sin(alpha) + (z-1-y0)*cos(alpha)) + 1 + y0;

%--------------------------------------------------
function [x y] = skalowanie(w, z, sx, sy, kierunek)

x = zeros(size(z,1),size(w,2));
y = zeros(size(z,1),size(w,2));

x = (w-1)*(sx^kierunek) + 1;
y = (z-1)*(sy^kierunek) + 1;

%--------------------------------------------------
function [x y] = translacja(w, z, dx, dy, kierunek)

x = zeros(size(z,1),size(w,2));
y = zeros(size(z,1),size(w,2));

x = (w-1) + (kierunek*dx) + 1;
y = (z-1) + (kierunek*dy) + 1;

%--------------------------------------------------
function args = parse_inputs(varargin)

iptchecknargin(1,Inf,nargin,mfilename);

args.wypelnienie = 0;
args.typ = [];
args.wymiar = [];
args.kierunek = 1;

args.X = check_X(varargin{1});

if rem(nargin,2) == 1
    first_prop_arg = 2;
    for k = first_prop_arg:2:nargin
        prop_string = check_property_string(varargin{k});
        switch prop_string
        case 'wypelnienie'
            args.wypelnienie = check_wypelnienie(varargin{k+1},args.X);  
        case 'wymiar'
            args.wymiar = check_wymiar(varargin{k+1}, args.X);
        case 'typ'
            args.typ = check_typ(varargin{k+1});
        case 'kierunek'
            args.kierunek = check_kierunek(varargin{k+1});
        otherwise
            eid = sprintf('Images:%s:internalError',mfilename);
            msg = sprintf('Unrecognized property string: %s', ...
                           prop_string);
            error(eid,'%s',msg);
        end;
    end
end

%--------------------------------------------------
function prop_string = check_property_string(prop_string)

if ~ischar(prop_string)
    eid = sprintf('Images:%s:invalidPropertyName',mfilename);
    msg = 'Property names must be strings.';
    error(eid,'%s',msg);
end
valid_strings = {'wypelnienie'
                 'wymiar'
                 'typ'
                 'translacja'
                 'skalowanie'
                 'obrot'
                 'Tform'
                 'kierunek'
                 'wprost'
                 'odwrotne'};
idx = strmatch(lower(prop_string), valid_strings);
switch length(idx)
 case 0
    eid = sprintf('Images:%s:unknownPropertyName',mfilename);
    msg = sprintf('Unrecognized property name: ''%s''', prop_string);
    error(eid,'%s',msg);
  case 1
    prop_string = valid_strings{idx};
  otherwise
    eid = sprintf('Images:%s:ambiguousPropertyName',mfilename);
    msg = sprintf('Xmbiguous property name: ''%s''', prop_string);
    error(eid,'%s',msg);
end

%--------------------------------------------------
function X = check_X(X)

if (~isnumeric(X) && ~islogical(X)) || issparse(X)
    eid = sprintf('Images:%s:invalidImage',mfilename);
    msg = 'X must be a nonsparse numeric array.';
    error(eid,'%s',msg);
end

%--------------------------------------------------
function wypelniene = check_wypelnienie(wypelniene, X)

if ~isnumeric(wypelnienie)
    eid = sprintf('Images:%s:invalidImage',mfilename);
    msg = 'Wypelnienie must be numeric.';
    error(eid,'%s',msg);
end

%--------------------------------------------------
function wymiar = check_wymiar(wymiar, typ, X)

if ~isnumeric(wymiar)
    eid = sprintf('Images:%s:invalidImage',mfilename);
    msg = 'Wymiar must be numeric array.';
    error(eid,'%s',msg);
end
if size(wymiar) == 1
    wymiar = [wymiar wymiar 0];
end

%--------------------------------------------------
function typ = check_typ(typ)

if ~ischar(typ)
    eid = sprintf('Images:%s:invalidImage',mfilename);
    msg = 'Typ must be a string.';
    error(eid,'%s',msg);
end

%--------------------------------------------------
function kierunek = check_kierunek(kierunek)

if ~ischar(kierunek)
    eid = sprintf('Images:%s:invalidImage',mfilename);
    msg = 'Kierunek must be a string.';
    error(eid,'%s',msg);
end
switch kierunek
    case 'wprost'
        kierunek = 1;
    otherwise
        kierunek = -1;
end;